require "config"
require "cachestat"
require "node"
require "nodelist"
require "node"
#require "signature"
require "util"

module Cache
  module_function
  #
  # add new data cache
  #
  def addData(file, newdata)
    Util.printtrace(file, newdata) ## debug

    return if newdata == []

    basename = "#{$conf["datadir"]}/#{file}"
    datas = (IO.readlines("#{basename}.dat") rescue [])
    datas.collect!{|x| x.chomp }

    Util.lock();

    datas += newdata
    datas.sort!

    str = ""
    records = 0
    begin
      File.open("#{basename}.dat.tmp", "w") {|f|
        datas.each {|line|
          if line != str then
            f << line << "\n"
            str = line
            records += 1
          end
        }
      }
    rescue Errno::ENOENT, Errno::EACCES
      raise "#{basename}.dat.tmp"
    end

    begin
      File.rename("#{basename}.dat.tmp", "#{basename}.dat")
    rescue
      raise "#{basename}: rename failed. #{$!}";
    end
    Util.unlock();

    newdata.each {|s|
      rec = Util.rec(s)
      if rec["remove_id"] && rec["remove_stamp"] then
        tmp = rec["target"] ? rec["target"].split(/,/) : []
        if tmp.include?("remove_stamp") && tmp.include?("remove_id") then
          if Signature.include(Signature.pubkey2trip(rec[:pubkey])) then
            if Signature.check(rec) then
              removeRecord(file, rec["remove_stamp"], rec["remove_id"])
            end
          end
        end
      end
    }

    s = datas[-1].split(/<>/)
    laststamp = s[0].to_i
    stat = CacheStat.list()
    stat[file] = {
        "stamp"   => laststamp,
        "records" => records,
        "size"    => FileTest.size("#{basename}.dat") || 0,
    }
    CacheStat.sync(stat)
  end

  #
  # get appointed data
  #
  def getData(file, stamp, id, node)
    Util.printtrace(file, stamp, id, node) ## debug

    newdata = node.talk("/get/#{file}/#{stamp.to_s}/#{id}");
    newdata = newdata.grep(/^[0-9]+<>[0-9A-Za-z]+<>/)
    newdata = [] if newdata.nil?
    newdata = Util.md5check(newdata)
    addData(file, newdata)
    return (newdata != 0)
  end

  #
  # get data in region
  #
  def getRegion(file, node)
    Util.printtrace(file, node) ## debug

    stat = CacheStat.list()
    basename = #{$conf["datadir"]}/#{file}

    xbegin = 0
    if stat[file] && stat[file]["stamp"] && FileTest.size?("#{basename}.dat") then
      xbegin = stat[file]["stamp"]
    end
    now = Time.now.to_i - ($conf["syncsafety"] * 60 * 60)
    xbegin = now if (now < xbegin)

    newdata = []
    if xbegin <= 0 then
      newdata = node.talk("/get/#{file}/0-")
      newdata = Util.md5check(newdata)
    else
      head = node.talk("/head/#{file}/#{xbegin}-");
      head.collect!{|x| x.chomp }

      tmp = []
      datas = (IO.readlines("#{basename}.dat") rescue [])
      datas.collect!{|x| x.chomp }

      head.each {|h|
        next if ! datas.grep(/^#{h}(<>|$)/).empty?

        stamp, id = h.split(/<>/)
        buf = node.talk("/get/#{file}/#{stamp}/#{id}")
        buf = Util.md5check(buf)
        Util.lock()
        begin
          File.open("#{basename}.dat", "a+") {|f|
            f << buf.join("\n") << "\n"
          }
        rescue
          raise "#{basename}: open failed. #{$!}"
        end
        Util.unlock()
      }
    end
    addData(file, newdata)
  end

  #
  # search node which has the file
  # args: file
  #
  def search(file)
    Util.printtrace(file) ## debug

    nodes  = NodeList.all()
    myself = NodeList.myself()
    myself = myself  ?  myself.toString() : ""

    done  = {}
    found = false
    node  = nil
    count = 0;

    while (! nodes.empty?) do
      node = nodes.shift
      buf = node.talk("/have/#{file}") # array
      if (buf.size >= 1) && (buf[0] == "YES") then
        found = true
        break
      elsif (count > $conf["searchDepth"]) then
        break
      else
        buf = node.talk("/node") # array
        if (buf.size <= 0) || (buf[0] == myself) then
          next
        end
        s = node.toString()
        done[s] = 0 if done[s].nil?
        unless (done[s] > $conf["retrySearch"]) then
          nodes.push(Node.new(buf[0]))
        end
        done[s] += 1
      end
      count += 1
    end
    return (found  ?  node : nil)
  end

  #
  # sync all files
  #
  def sync()
    Util.printtrace() ## debug

    files = Dir.glob("#{$conf[:datadir]}/*.dat")

    files.size.downto(1) {|i|
      t = rand(i)
      file[t-1], file[t] = file[t], file[t-1]
    }

    files.each {|name|
      Util.debug("#{name}.size = ", FileTest.size(name).to_s)  ## debug
      if FileTest.size(name) <= ($conf["filelimit"] * 1024 * 1024) then
        name.sub!(/#{$conf["datadir"]}\//,'')
        name.sub!(/\.dat$/, '')
        node = search(name) and getRegion(name, node)  #
      end
    }
  end

  #
  # tell update other nodes
  # if $node==null then $node := myself
  #
  def tellupdate(file, stamp, id, node)
    Util.printtrace(file, stamp, id, node) ## debug

    nodes = NodeList.all()
#   pid = fork {
    # child process
#     Signal.trap("TERM", "DEFAULT")
#     Signal.trap("HUP",  "DEFAULT")
#     STDOUT.close

      node = NodeList.myself() if node.nil?

      if node then
        node_str = node.toXString()
        nodes.each {|node|
          node.talk("/update/#{file}/#{stamp}/#{id}/#{node_str}")
        }
      end
#    exit
#  }
  end

  #
  # make new files writable
  #
  def newFiles()
    Util.printtrace() ## debug

    names = Dir.glob("#{$conf["datadir"]}/*.new")
    names.each {|name|
      new = name
      name.sub!(/\.new$/, '')
      File.rename(new, name)
      file = name
      file.sub!(/^#{$conf["datadir"]}\//, '')
      file.sub!(/\.dat$/, '')
      stat = CacheStat.list()
      stat[file] = {
          "stamp"   => 0,
          "records" => 1,
          "size"    => FileTest.size?("#{$conf["datadir"]}/#{file}.dat") || 0,
        }
      CacheStat.sync(stat)
    }
  end

  #
  # remove record
  #
  def removeRecord(file, stamp, id)
    Util.printtrace(file, stamp, id) ## debug

    basename = "#{$conf["datadir"]}/#{file}"
    r = nil
    w = nil
    begin
      r = File.open("#{basename}.dat", "r")
    rescue
      return false
    end

    Util.lock()

    begin
      w = File.open("#{basename}.dat.tmp", "w")
    rescue
      raise "#{basename}.dat.tmp: failed to open. #{$!}"
    end

    r.each_line {|line|
      if line =~ /^#{stamp}<>#{id}<>/ then
        w << "#{stamp}<>#{id}\n"
      else
        w << line
      end
    }
    w.close
    r.close

    begin
      File.rename("#{basename}.dat.tmp", "#{basename}.dat")
    rescue
      raise "#{basename}: rename failed. #{$!}"
    end
    Util.unlock()

    return true
  end
end
