#!/usr/bin/ruby

require "config"
require "nodelist"
require "cache"
require "cachestat"
require "gateway"
require "message"
require "signature"
require "util"

class GatewayCGI < Gateway
  def initialize
    super
    $conf["debugid"] = "gtwy" # debug
    @types = %w(list thread note)
    @root  = $conf["root"]
  end

  #
  # main routine
  #
  def execute(path, input)
    Util.printtrace(path, input) ## debug

    File.umask($conf["umask"])

    arg = args(input)
    cmd = arg["cmd"] || ""

    if cmd == "new" then
      xdie($message["null_name"]) if arg["name"] == ""
      xdie($message["bad_name" ]) if arg["name"] =~ /\//
      xdie($message["bad_name" ]) if arg["name"] =~ /\]\]/
      xdie($message["null_type"]) if arg["type"] == ""
      if arg["type"] =~ /^(list|thread|note)$/ then
        print302("#{@root}#{arg["type"]}.cgi/" + str_encode(arg["name"]))
      else
        print404()
      end

    elsif path == "delete" then
      checkAdmin()
      deleteDialog(arg)

    elsif cmd == "xdelete" then
      if arg["file"].nil? then
        print404()
        exit
      elsif arg["stamp"] && arg["id"] then
        deleteRecord(arg["file"], arg["stamp"], arg["id"], arg)
      else
        deleteFile(arg["file"])
      end

    elsif (path == "") || (path == "title") then
      printTitle()

    elsif path == "list" then
      print302("#{@root}list.cgi/")

    elsif path == "motd" then
      printMotd()

    elsif path =~ %r!^(list|thread|note)/([^/]+)$! then
      print302("#{@root}#{$1}.cgi/" + str_encode($2))

    elsif path == "new" then
      printHeader(@message["new"])
      newElementForm()
      @body << "</body></html>\n"

    elsif path == "index" then
      checkAdmin()
      printIndex()

    elsif path == "changes" then
      checkAdmin()
      printChanges()

    elsif path == "update" then
      checkAdmin()
      printUpdate()

    elsif path =~ /^search/ then
      query = $'
      checkAdmin()
      if query && (query =~ /^\/(.+)/) then
        printSearchResult($1)
      elsif input && input != "" then
        arg = args(input)
        printSearchResult(arg["query"])
      else
        printHeader(@message["search"])
        searchForm()
        @body << "</body></html>"
      end

    elsif path == "edittrust" then
      checkAdmin()
      arg = args(input)
      if arg["trip"].nil? then
        print404()
        exit
      elsif arg["button"] == $message["add"] then
        Signature.addlist(arg["trip"])
      elsif arg["button"] == $message["delete"] then
        Signature.dellist(arg["trip"])
      end
      if arg["type"] =~ /^(list|thread|note)$/ then
        print302("#{@root}#{arg["type"]}.cgi/" + str_encode(file_decode("#{arg["type"]}_#{arg["file"]}")))
      end

    else
      print404()
    end

    return [@head, @body]
  end

  #
  # print title page
  #
  def printTitle()
    Util.printtrace() ## debug

    printHeader(@message["logo"])
    @body << "<p>#{@message["license"]}</p>\n" <<
             "<ul><li><em><a href='#{@root}list.cgi'>#{@message["menu"]}</a></em></li>\n" <<
             "<li><a href='http://shingetsu.sourceforge.net/'>" <<
             "#{@message["site"]}</a></li>\n" <<
             "<li><a href='#{ENV["SCRIPT_NAME"]}/motd'>#{@message["agreement"]}</a></li></ul>" <<
             "</body></html>\n"
  end

  #
  # print MOTD file
  #
  def printMotd()
    Util.printtrace() ## debug

    @head << "Content-type: text/plain; charset=euc-jp\r\n"   <<
             "X-Shingetsu: #{$conf["version"]}\r\n" <<
             "\r\n"
    IO.foreach($conf["motd"]) {|line|
      @body << line
    }
  end

  #
  # print index page
  #
  def printIndex()
    Util.printtrace() ## debug

    printHeader(@message["index"])
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}/delete'>\n"

    stat = CacheStat.list() # Hash
    @types.each {|type|
      @body << "<h2>#{@message[type]}</h2><ul>\n"
      Dir.glob("#{@datadir}/#{type}_*.dat").sort.each {|fname|
        fname.sub!(/^#{@datadir}\//, "")
        fname.sub!(/.dat$/, "")
        x = file_decode(fname)
        y = str_encode(x)
        records =  stat[fname]["records"]
        size    = (stat[fname]["size"]/1024/1024*10).to_i / 10
        @body << "<li><input type='radio' name='file' value='#{fname}' tabindex='1' accesskey='s' />" <<
                 "<a href='#{@root}#{type}.cgi/#{y}'>#{x}</a>(#{records}/#{size.to_s}#{@message["mb"]})</li>\n"
      }
      @body << "</ul>"
    }
    @body << "<p><input type='submit' value='#{@message["del_file"]}' tabindex='2' accesskey='d' /></p></form>"
    newElementForm()
    @body << "</body></html>\n"
  end

  #
  # print recent changes
  #
  def printChanges()
    Util.printtrace() ## debug

    printHeader(@message["changes"])
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}/delete'>\n"

    stat = CacheStat.list()
    @types.each {|type|
      @body << "<h2>#{@message[type]}</h2><ul>\n"
      bufs = []
      stamp = {}
      Dir.glob("#{@datadir}/#{type}_*.dat") {|fname|
        fname.sub!(/^#{@datadir}\//, "")
        fname.sub!(/.dat$/, "")
        x    = file_decode(fname)
        y    = str_encode(x)
        date = xlocaltime(stat[fname]["stamp"])
        records =  stat[fname]["records"]
        size    = (stat[fname]["size"]/1024/1024*10).to_i / 10

        buf = "<li><input type='radio' name='file' value='#{fname}' tabindex='1' accesskey='s' />#{date} <a href='#{@root}#{type}.cgi/#{y}'>#{x}</a>(#{records}/#{size.to_s}#{@message["mb"]})</li>\n"
        bufs << [date.to_i, buf]
      }
      bufs.sort! {|a,b| b[0] <=> a[0]}
      bufs.each  {|x|   @body << x[1]}
      @body << "</ul>"
    }
    @body << "<p><input type='submit' value='#{@message["del_file"]}' tabindex='2' accesskey='d' /></p></form>"
    newElementForm()
    @body << "</body></html>\n"
  end

  #
  # print recent update
  #
  def printUpdate()
    Util.printtrace() ## debug

    stamp = {}
    IO.foreach($conf["updatelist"]) {|line|
      line.chomp!
      items = line.split(/<>/)
      if items[2] =~ /^(list|thread|note)_/ then
        stamp[items[2]] = items[0]
      end
    }
    printHeader(@message["update"])

    stat = CacheStat.list() # Hash
    @types.each {|type|
      @body << "<h2>#{@message[type]}</h2><ul>\n"
      arr = stamp.keys.grep(/^#{type}_/).sort {|a,b| stamp[b] <=> stamp[a]}
      arr.each {|key|
        x    = file_decode(key)
        y    = str_encode(x)
        date = xlocaltime(stamp[key])
        records = stat[key] ?   stat[key]["records"].to_s : "?"
        size    = stat[key] ? ((stat[key]["size"]/1024/1024*10).to_i / 10).to_s : "?"
        @body << "<li>#{date} <a href='#{@root}#{type}.cgi/#{y}'>#{x}</a>(#{records}/#{size}#{@message["mb"]})</li>\n"
      }
      @body << "</ul>"
    }
    newElementForm()
    @body << "</body></html>\n"
  rescue Errno::ENOENT, Errno::EACCES
    print404(@message["update"])
  end

  #
  # search form
  #
  def searchForm(arg = nil)
    Util.printtrace(arg) ## debug

    query = arg || ""
    xquery = query.dup
    xquery.gsub!(/\"/, "&quot;")
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}/search'><p>" <<
             "<input name='query' size='19' value=\"#{xquery}\" tabindex='1' accesskey='q' />" <<
             "<input type='submit' value='#{@message["search"]}' name='submit' tabindex='1' accesskey='s' />" <<
             "</p></form>"

#   query.gsub!(/\|/, " OR ")
#   query.gsub!(/\.[+*]?/, " ")
    query.gsub!(/"/, "&quot;")
    @body << "<form method='get' action='#{$conf["google"]}'><p>" <<
             "<input name='q' size='19' value=\"#{query}\" tabindex='1' accesskey='g' />" <<
             "<input type='hidden' name='ie' value='UTF-8' />" <<
             "<input type='submit' name='google' value='#{@message["google"]}' tabindex='1' accesskey='g' />" <<
             "</p></form>"
  end

  #
  # search result
  #
  def printSearchResult(query)
    Util.printtrace(query) ## debug

    printHeader(@message["search"])
    @body << "\n"

    stat = CacheStat.list()
    @types.each {|type|
      @body << "<h2>#{@message["type"]}</h2><ul>\n"
      files = search(type, query) # Hash
      files.each {|file|
        x = file_decode(file)
        y = str_encode(x)
        records =  stat[file]["records"]
        size    = (stat[file]["size"]/1024/1024*10).to_i / 10
        @body << "<li><a href='#{@root}#{type}.cgi/#{y}'>#{x}</a>(#{records}/#{size.to_s}#{@message["mb"]})</li>\n"
      }
      @body << "</ul>"
    }
    searchForm(query)
    @body << "</body></html>"
  end
end
