require "config"
require "cache"
require "cachestat"
require "gateway"
require "message"
require "util"

class ThreadCGI < Gateway
  def initialize
    super
    $conf["debugid"] = "thrd" # debug
  end

  #
  # main routine
  #
  def execute(path, input)
    Util.printtrace() ## debug

    File.umask($conf["umask"])

    if ENV["CONTENT_TYPE"] && (ENV["CONTENT_TYPE"] =~ /multipart\/form-data/) then
      arg = argsFromMulti(input)
    else
      arg = args(input)
    end

    cmd    = arg["cmd"   ] || ""
    file   = arg["file"  ] || ""
    record = arg["record"] || ""

    if (cmd == "post") && (file =~ /^thread_[0-9A-F]+$/) then
      id = post(file, arg)
      print302("#{@root}thread.cgi/" + str_encode(file_decode(file)) + "#r#{id}")

    elsif cmd == "check" then
      checkSign(arg)

    elsif cmd == "delete"
      checkAdmin()
      deleteDialog(arg)

    elsif cmd == "xdelete" then
      checkAdmin()
      if arg["file"].nil? then
        print404()
        exit
      elsif arg["stamp"] && arg["id"] then
        deleteRecord(arg["file"], arg["stamp"], arg["id"], arg)
      else
        deleteFile(arg["file"])
      end

    elsif (path != "") && (path !~ /\//) then
      printThread(path)

    elsif path =~ %r!^([^/]+)/([0-9a-f]{32})/(\d+)\.(.*)! then
      file, stamp, id, suffix = $1, $2, $3, $4
      printAttach(file, stamp, id, suffix)

    else
      print404()
      exit
    end
  end
  #
  # print thread
  #
  def printThread(thread)
    Util.printtrace(thread) ## debug

    thread = escape(thread)
    file_thread = file_encode("thread", thread)
    touch(file_thread)

    begin
      f = File.open("#{@datadir}/#{file_thread}.dat", "r")
      printHeader(thread)
      @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
               "<p><input type='hidden' name='cmd' value='delete'  />" <<
               "<input type='hidden' name='file' value='#{file_thread}'  /></p><dl>\n"

      count = 0
      f.each_line {|line|
        count += 1
        rec = rec(line) # Hash
        next if rec.size <= 2
        xstamp = xlocaltime(rec["stamp"])
        rec["name"] = @message["anonymous"] if rec["name"].nil? || (rec["name"] == "")
        mail = ""
        mail = "[#{rec["mail"]}]" if rec["mail"]
        rec["body"] = "" if rec["body"].nil?
        rec["body"] = html_format(rec["body"])
        if $conf["redirect"] then
          rec["body"].gsub!(/<a href='http:\/\/shingetsu\.p2p\//, "<a href='/")
        end
        attach = ""
        check  = ""
        remove = ""

        if rec["attach"] then
          size = (rec["attach"].size / 4 * 3 / 1024).to_i
          attach = " <a href='#{ENV["SCRIPT_NAME"]}" +
              str_encode(thread) + "/#{rec["id"]}/#{rec["stamp"]}.#{rec["suffix"]}'>" +
              "#{rec["stamp"]}.#{rec["suffix"]}</a> (#{size}#{@message["kb"]})"
        end
        if rec["pubkey"] then
          check = "<a href='#{ENV["SCRIPT_NAME"]}?cmd=check&amp;" +
             "file=" + file_encode("thread", thread) + "&amp;" +
             "stamp=#{rec["stamp"]}&amp;id=#{rec["id"]}'>" +
             Signature::pubkey2trip(rec["pubkey"]) + "</a>"
        end
        if rec["remove_id"] && rec["remove_stamp"] then
          xid = rec["remove_id"][0, 8] # substr
          remove = "[[#{@message["remove"]}: <a href='#r#{xid}'>#{xid}</a>]]<br />"
        end
        xid = rec["id"][0, 8] # substr
        @body << "<dt id='r#{xid}'><input type='radio' name='record' value='#{rec["stamp"]}/#{rec["id"]}' " <<
                 "tabindex='1' accesskey='s' />" <<
                 "#{xid} <span class='name'>#{rec["name"]}</span>" <<
                 " #{mail} #{check} #{xstamp}#{attach}</dt><dd>#{rec["body"]}<br />#{remove}<br /></dd>\n"
      }
      f.close
    rescue Errno::ENOENT, Errno::EACCES
      raise "#{@datadir}/#{file_thread}.dat: failed to open. #{$!}#{$@}"
    end

    @body << "</dl><p><input type='submit' value='#{@message["del_record"]}' tabindex='2' accesskey='d' />" <<
             "<input type='hidden' name='mode' value='thread' /></p></form>"

    filesize = (FileTest.size("#{@datadir}/#{file_thread}.dat")/1024/1024*10).to_i / 10
    checked = count ? "checked='checked'" : ""
    readonly = (ENV["REMOTE_ADDR"] =~ /#{$conf["adminAddr"]}/) ? "" : "readonly='readonly'"

    if filesize <= $conf["filelimit"] then
      @body << "<form method='post' action='#{ENV["SCRIPT_NAME"]}' enctype='multipart/form-data'><p>" <<
               "<input type='hidden' value='post' name='cmd' />" <<
               "<input type='hidden' value='#{file_thread}' name='file' />" <<
               "<input type='submit' value='#{@message["post"]}' name='submit' tabindex='1' accesskey='w' />" <<
               " <input type='checkbox' value='dopost' name='dopost'" <<
               " #{checked} tabindex='2' accesskey='s' />" <<
               "#{@message["send"]}" <<
               " <input type='checkbox' value='error' name='error'" <<
               " checked='checked' tabindex='2' accesskey='e' />" <<
               "#{@message["error"]}<br />" <<
               " #{@message["name"]}:<input name='name' size='15' value='' tabindex='3' accesskey='n' />" <<
               " #{@message["mail"]}:<input name='mail' size='15' value='' tabindex='4' accesskey='m' />" <<
               " #{@message["signature"]}:<input type='password' name='passwd' size='15' #{readonly} value='' tabindex='5' accesskey='p' /><br />" <<
               " #{@message["attach"]}:<input type='file' name='attach' size='19' value='' tabindex='6' accesskey='a' />" <<
               " #{@message["suffix"]}:<select name='suffix' size='1' tabindex='7'>" <<
               "<option>AUTO</option>"
      $conf["mimeType"].keys.sort.each {|type|
        @body << "<option>#{type}</option>"
      }
      @body << "</select> (#{@message["limit"]}: #{$conf["filelimit"]}#{@message["mb"]})<br />" <<
               "<textarea rows='5' cols='70' name='body' tabindex='8' accesskey='c'></textarea>" <<
               "</p></form>"
    end

    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='submit' value='#{@message["del_file"]}' tabindex='9' accesskey='d' />" <<
             "<input type='hidden' name='cmd' value='delete' />" <<
             "<input type='hidden' name='file' value='#{file_thread}' />" <<
             " #{filesize}#{@message["mb"]}</p></form></body></html>\n"
  end

  #
  # print attachment
  #
  def printAttach(thread, id, stamp, suffix)
    Util.printtrace(thread, id, stamp, suffix) ## debug

    thread = file_encode("thread", escape(thread))
    suffix.downcase!

    type = $conf["mimeType"][suffix]
    type = "text/plain" if type.nil?
    touch(thread)

    value = nil
    IO.foreach("#{@datadir}/#{thread}.dat") {|line|
      rec = rec(line)
      if rec["attach"] && (rec["stamp"] == stamp) && (rec["id"] == id) then
        value = rec["attach"]
        break
      end
    }

    if value then
      @head << "Content-Type: #{type}\r\n" <<
               "X-Shingetsu: #{$conf["version"]}\r\n" <<
               "\r\n"
      @body << Extern.base64decode(value)
    else
      print404(thread)
    end
  rescue Errno::ENOENT, Errno::EACCES
    raise "#{@datadir}/#{thread.dat}: failed to open. #{$!}#{$@}"
  end
end
