
require "config"
require "extern"

#
# configuration
#

module Util
  module_function

  def printtrace(*msg)
    return unless $conf["debug"]
    s = caller(1)
    fr = s[1]
    s[1] =~ /\.\/([^.]+)\D+(\d+):in \`(\w+)/
    fr = "#{$1}.#{$3}(#{$2})"
    s[0] =~ /\.\/([^.]+)\D+(\d+):in \`(\w+)/
    to = "#{$1}.#{$3}(#{$2})  "
    arg = ""
    msg.each{|x| arg += x.inspect[0..200] + ', '}
    n = (s.size < 5) ? 0 : s.size - 5
    STDERR.print "  #{$conf["debugid"]} : #{" " * n}\033[1;35m#{fr} --> #{to}\033[0m #{arg}\n"
    STDERR.flush
  end

  def debug(msg, *arg)
    return unless $conf["debug"]
    s = ""
    arg.each{|x| s += x.inspect[0..200] + ', '}
    STDERR.print "  #{$conf["debugid"]} : #{msg}#{s}\n"
    STDERR.flush
  end

  def alarm(sec)
    if RUBY_PLATFORM =~ /(linux)/ then
      syscall(27, sec)  # 27 = SYS_alarm (i386-Linux)
    end
  end

  $lock_f = 0

  def lock()
    if RUBY_PLATFORM !~ /cygwin/ then
      $lock_f = File.open("#{$conf["lock"]}", "w")
      $lock_f.flock(File::Constants::LOCK_EX)
    end
  end

  def unlock()
    if RUBY_PLATFORM !~ /cygwin/ then
      $lock_f.close
    end
  end

  #
  # add a line to update list
  # args: stamp, id, filename
  #
  def addUpdate(stamp, id, filename)
    updatelist = $conf["updatelist"]
    saveupdate = $conf["saveupdate"]

    lock()

    update = (IO.readlines(updatelist) rescue [])
    update << "#{stamp}<>#{id}<>#{filename}\n"

    now = Time.now.to_i

    begin
      File.open(updatelist, "w") {|f|
        update.each {|line|
          items = line.split(/<>/)
          if (now - items[0].to_i) <= (saveupdate * 60) then
            f << line
          end
        }
      }
    rescue
      raise "#{updatelist}: failed to open. #{$!}"
    end
  ensure
    unlock()
  end

  #
  # normal random by Box-Muller Transformation
  #
  def normal()
    include Math
    r1, r2 = rand, rand
    return Math.sqrt(-2 * Math.log(r1)) * Math.sin(2.0 * 3.141592653 * r2)
  end

  #
  # time with error
  #
  def time()
    error = $conf["timeError"] * normal()
    return Time.now.to_i + error.to_i
  end

  #
  # localtime wrapper for Cygwin's bug
  #
  def localtime(sec = nil)
    if sec then
      return Time.at(sec).localtime.to_i
    else
      return Time.now.localtime.to_i
    end
  end

  #
  # MD5 check records
  #
  def md5check(record)
    buf = []
    record.each {|line|
      items = line.split(/<>/, 3)  # limit 3 times
      if items.size < 3 then
        next
      elsif (items[0] =~ /^[0-9]+$/) &&
            (items[1] == Extern.md5digest(items[2])) then
        buf << line
      end
    }
    return buf
  end

  #
  # parse cache record
  #
  def rec(arg)
    items = arg.chomp.split(/<>/)
    hash = {"stamp" => items.shift,
            "id"    => items.shift }

    items.each {|s|
      bufs = s.split(/:/, 2)  # limit 2 times
      next if bufs.size < 2
      bufs[1].gsub!(/<br>/, "\n"  )
      bufs[1].gsub!(/</,    "&lt;")
      bufs[1].gsub!(/>/,    "&gt;")
      bufs[1].gsub!(/\n/,   "<br>")
      hash[ bufs[0] ] = bufs[1]
    }
    return hash
  end

  #
  # wget
  #
  def wget(uri)
#   printtrace() ## debug
    return nil if uri !~ /http:\/\/(.+):(\d+)\/(.*)/

    host, port, path = $1, $2, "/#{$3}"
    timeout = $conf["timeout"]
    bufsize = $conf["bufsize"]

    inn, out = IO.pipe

    fork {
      # child process
      Signal.trap("ALRM") { 
        out.close
        raise "#{host}:#{port}: connection timeout."
      }
      inn.close

      begin
        require "socket"
        sock = TCPSocket.open(host, port)

        alarm(timeout)
        sock.print "GET #{path} HTTP/1.0\r\n",
              "User-Agent: #{$conf["version"]}\r\n",
              "Host: #{host}:#{port}\r\n",
              "Accept: text/plain\r\n",
              "Accept-Encoding: gzip\r\n",
              "Connection: close\r\n",
              "\r\n\r\n"
        buf = " " * bufsize
        while sock.read(bufsize, buf) do
          out << buf
          alarm(timeout)
        end
      rescue
        raise "#{host}:#{port}: connection refused. #{$!}"
      ensure
        alarm(0)
        sock.close rescue nil
        out.close  rescue nil
        exit
      end
    }

    # parent process
    buf = []
    Signal.trap("CHLD", "DEFAULT")
    out.close
    inn.each_line {|line|
      buf << line
    }
    return buf
  end
end

